/*-------------------------------------------------------------*/
/*   x11intf: definition of actions                            */
/*-------------------------------------------------------------*/
/*                                                             */
/* action                                                      */
/* -1 : Store config-file name in static array                 */
/*  0 : close the window                                       */
/*  1 : open the window                                        */
/*  2 : draw from x1, y1 to x2, y2                             */
/*  3 : Rings the Bell                                         */ 
/*  4 : manage the mouse                                       */
/*  5 : define colors in blocks of 3 rgb values                */
/*  6 : clear screen                                           */
/*  7 : xflush                                                 */
/*  8 : Define ViewPort                                        */
/*  9 : draw string (z chars) in position x,y                  */
/* 10 : draw circle center in  x,y ,radius = z                 */
/* 11 : fill circle center in  x,y ,radius = z                 */
/* 12 : set foreground color to col                            */
/* 13 : draw z-points polygon using color col                  */
/* 14 : fill z-points polygon using color col                  */
/* 15 : Print a message at the bottom of the window            */
/* 16 : Store atomic species in static array                   */
/* 17 : Draw rectangle with col filled with col1               */
/* 18 : Remove a file                                          */
/* 19 : Save(Restore) Area#1    iz=1(2)                        */
/* 20 : Save(Restore) Area#2    iz=1(2)                        */
/* 21 : Save(Restore) Area#3    iz=1(2)                        */
/* 22 : Save(Restore) Area#4    iz=1(2)                        */
/* 23 : Save(Restore) Area#5    iz=1(2)                        */
/* 24 : Save(Restore) Area#6    iz=1(2)                        */
/* 25 : Save(Restore) Area#7    iz=1(2)                        */
/* 26 : Save(Restore) Area#8    iz=1(2)                        */
/* 27 : Save(Restore) Area#9    iz=1(2)                        */
/* 29 : Connect iz points using color = icol                   */
/* 30 : return in xv[0] the number of pix in string; iz=len(str*/
/* 44 : change font iz=1 -> proportional  iz=2 -> Non propor.  */
/* 61 : Save(Restore) Area#1    iz=1(2)  if NO backingstore    */
/* 62 : Save(Restore) Area#2    iz=1(2)  if NO backingstore    */
/* 63 : Save(Restore) Area#3    iz=1(2)  if NO backingstore    */
/* 64 : Save(Restore) Area#4    iz=1(2)  if NO backingstore    */
/* 65 : Save(Restore) Area#5    iz=1(2)  if NO backingstore    */
/* 66 : Save(Restore) Area#6    iz=1(2)  if NO backingstore    */
/* 67 : Save(Restore) Area#7    iz=1(2)  if NO backingstore    */
/* 68 : Save(Restore) Area#8    iz=1(2)  if NO backingstore    */
/* 69 : Save(Restore) Area#9    iz=1(2)  if NO backingstore    */
/* 80 : Change cursor shape                                    */
/* 90 : draw string (z chars) in position x,y                  */
/*-------------------------------------------------------------*/

/*-------------------------------------------------------------*/
/*          Include files                                      */
/*-------------------------------------------------------------*/
/* CHANGE  following include to be used ONLY on VMS computers  */
/* OTHERWISE use standard X11 include                          */

/*
#include "sys$common:[decw$include]Xlib.h"
#include "sys$common:[decw$include]Xutil.h"
#include "sys$common:[decw$include]Xos.h"
#include "sys$common:[decw$include]Intrinsic.h"
#include "sys$common:[decw$include]Xatom.h"
*/


#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>

#include <stdio.h>
#include <stdlib.h>

/*-------------------------------------------------------------*/
/*          Constants  definition                              */
/*-------------------------------------------------------------*/
#define MAX_COLORS  32
#define MAX_FONTLEN 120
#define MAX_FONTNUM 3000
#define PITCHP      10	
#define PITCHNP     10	

/*-------------------------------------------------------------*/
/*          Functions declaration                              */
/*-------------------------------------------------------------*/
void C_PrintMess ();
void PrintImageMess ();
void Bottom();
void Sorcio_C ();
XFontStruct *LoadFont  ();
XFontStruct *LoadFontGen  ();
static Cursor CursorID;
static XRectangle drawing,all;
void crea_listafonts();
void update_listafonts();

/*-------------------------------------------------------------*/
/*      Static variables (to save value)                       */
/*-------------------------------------------------------------*/
static char Titolo[81],filcon[81];
static char Species[81];
static char Species_copy[81];
static int BackStore, depthW, kkprog;
static int NumSpec,Ascent,Descent;
static int VP_x,VP_y,VP_w,VP_h;
static char fontp[MAX_FONTLEN], fontn[MAX_FONTLEN];
static int Saved1_x, Saved1_y, Saved1_w, Saved1_h;
static int Saved2_x, Saved2_y, Saved2_w, Saved2_h;
static int Saved3_x, Saved3_y, Saved3_w, Saved3_h;
static int Saved4_x, Saved4_y, Saved4_w, Saved4_h;
static int Saved5_x, Saved5_y, Saved5_w, Saved5_h;
static int Saved6_x, Saved6_y, Saved6_w, Saved6_h;
static int Saved7_x, Saved7_y, Saved7_w, Saved7_h;
static int Saved8_x, Saved8_y, Saved8_w, Saved8_h;
static int Saved9_x, Saved9_y, Saved9_w, Saved9_h;
static int  ClipX, ClipY, ClipN ;
static int  itwas31, itwas31y ;
static Display *display;
static GC     gc;                  
static Boolean Monocro = FALSE;
static Boolean hc = FALSE;
static int colors[MAX_COLORS],white,black,old_color,new_color;
static int BW = 20 , BH = 50 ;
static int jscreen = 0;
static int deltax,deltay,kscrx,kscry;
static int iHConv;
static int width, height,wi,he;             
static int ifProportional, ifTwoFonts ;
static Window mwindow;
static SavedArea1, SavedArea2, SavedArea3;
static SavedArea4, SavedArea5, SavedArea6;
static SavedArea7, SavedArea8, SavedArea9;
static Pixmap SavedPixM1, SavedPixM2, SavedPixM3 ;
static Pixmap SavedPixM4, SavedPixM5, SavedPixM6 ;
static Pixmap SavedPixM7, SavedPixM8, SavedPixM9 ;
static XFontStruct *fontPunt, *fontStruct, *fontViewer;
static Pixmap Icona;

#define icosir_width 32
#define icosir_height 32
static unsigned char icosir_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x03, 0x0e, 0x00, 0xf8, 0x0f, 0x0e, 0x00,
   0xfc, 0x0f, 0x0e, 0x00, 0x1e, 0x1e, 0x00, 0x00, 0x0e, 0x1c, 0x0e, 0x77,
   0x1e, 0x00, 0x0e, 0x7f, 0xfe, 0x00, 0x0e, 0x7f, 0xfc, 0x07, 0x0e, 0x0f,
   0xf0, 0x1f, 0x0e, 0x07, 0x80, 0x1f, 0x0e, 0x07, 0x00, 0x3c, 0x0e, 0x07,
   0x06, 0x38, 0x0e, 0x07, 0x0e, 0x38, 0x0e, 0x07, 0x1e, 0x3c, 0x0e, 0x07,
   0xfe, 0x10, 0x06, 0x00, 0x3c, 0x0f, 0xf6, 0x1f, 0xd0, 0x3f, 0xf6, 0x1f,
   0xc0, 0x39, 0x00, 0x1c, 0xe0, 0x70, 0x00, 0x0e, 0xe0, 0x70, 0x00, 0x07,
   0xe0, 0x70, 0x00, 0x07, 0xe0, 0x70, 0x80, 0x03, 0xc0, 0x79, 0x80, 0x03,
   0xc0, 0x7f, 0xc0, 0x01, 0x80, 0x77, 0xc0, 0x01, 0x00, 0x70, 0xc0, 0x01,
   0xc0, 0x70, 0xe0, 0x00, 0xe0, 0x39, 0xe0, 0x00, 0xc0, 0x3f, 0xe0, 0x00,
   0x80, 0x0f, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00};

#define icoexp_width 32
#define icoexp_height 32
static unsigned char icoexp_bits[] = {
   0x00, 0xc0, 0x01, 0x0e, 0x00, 0xc0, 0x01, 0x0f, 0x00, 0x80, 0x83, 0x07,
   0x00, 0x80, 0xc3, 0x03, 0x00, 0x00, 0xe7, 0x01, 0xe0, 0x03, 0xf7, 0x00,
   0xf8, 0x07, 0x7e, 0x00, 0x1c, 0x0e, 0x3e, 0x00, 0x0c, 0x0e, 0x1c, 0x00,
   0xfe, 0x0f, 0x3e, 0x00, 0xfe, 0x0f, 0x3f, 0x00, 0x0e, 0x80, 0x77, 0x00,
   0x0e, 0xc0, 0x73, 0x00, 0x1c, 0xe0, 0xe1, 0x00, 0xfc, 0xf3, 0xe0, 0x00,
   0x70, 0x00, 0x40, 0x00, 0x00, 0xf7, 0x80, 0x0f, 0x00, 0xff, 0xe1, 0x3f,
   0x00, 0xc7, 0xf3, 0x78, 0x80, 0x83, 0x73, 0x70, 0x80, 0x83, 0x3b, 0x70,
   0x80, 0x83, 0x3b, 0x70, 0x80, 0x83, 0x3b, 0x70, 0x80, 0xc3, 0x39, 0x38,
   0xc0, 0xe7, 0x71, 0x3c, 0xc0, 0xff, 0xf0, 0x1f, 0xc0, 0x7d, 0xc0, 0x07,
   0xc0, 0x01, 0x00, 0x00, 0xe0, 0x00, 0x00, 0x00, 0xe0, 0x00, 0x00, 0x00,
   0xe0, 0x00, 0x00, 0x00, 0xe0, 0x00, 0x00, 0x00};

/*-------------------------------------------------------------*/
/*      Global variables declaration                           */
/*-------------------------------------------------------------*/
XPoint points[100], Points4[4];
XEvent event;

/*-------------------------------------------------------------*/
/*   x11intf                      X11 interface routine        */
/*-------------------------------------------------------------*/
/*   CHANGE       for Ultrix or Silicon Graphics  or Linux     */
/*   void x11intf_(xv,yv,z, action, col )                      */
/*   OTHERWISE                                                 */
/*   void x11intf(xv,yv,z, action, col )                       */
/*-------------------------------------------------------------*/
void x11intf(xv,yv,z, action, col , col1 , string)
int xv[],yv[],*z,*action;
int *col , *col1 ;
   char string[];


{

   unsigned int stip_width, stip_height;
   unsigned int Shape;
   char *filename = "maschera.inp";

   int *x, *y,np,iact,MouseExitCode;
   XGCValues values;

   static int Angle1 = 23040 ;
   static int Angle2 = 23040 ;

   static int xlx, yly, dx, dy ;         /* more statics for     */
   static int start_xlx, start_yly;      /* line drawing         */
   static int mouse_x, mouse_y;      /* mouse coordinates    */
   static int larSaveN, altSaveN, larSaveP, altSaveP ;
   static int widb , heib;
   static Boolean loop0,loop1;
   FILE *fp;
   char rigar[MAX_FONTLEN];
   static char nome[256];

   int count,n,ix,iy;
   int xb,yb,i,j,ii;
   int Iz;
   XSetWindowAttributes attr;
   XSizeHints hints;
   XWMHints wmhints;
   char messaggio[80];
   int asc,des,lar,alt,larb,altb,larmin;
   int tsx, tsy, fill_style;
   XGCValues valore;
   int chfont;

   x = xv;
   y = yv;
   itwas31y = *y;
   itwas31 = 0;
   if ( *action == 31 )
      {
      *action = 15;
      itwas31 = 1;
      }
   if ( *action >= 19 && *action <= 27 && BackStore != 2 )
      {
      *action = *action + 42;
      }
   switch (*action)
   {
   case -1:
      deltax = xv[0];
      deltay = xv[1];
      kscrx  = xv[2];
      kscry  = xv[3];
      np = xv[4];
      sprintf(nome, "%s", string);
      nome[np] = '\0';
	fprintf(stderr," Configuration file: %s\n",nome);
	fp = fopen(nome, "r");
        if (fp == NULL)
		fprintf(stderr," Error in open file: %s\n",nome);
	else
	{
        for (np=1;np<7;np++)
            {
	      if(fgets(rigar,MAX_FONTLEN,fp) == NULL)
	      	fprintf(stderr,"Error in fgets\n");
            }
	fclose(fp);
        }
      break;

   case 0:                               /* close the window     */
      XFreeGC(display,gc);
      XCloseDisplay(display);
      break;

   case 1:                               /* open or resize Xwindow  */
      if ( jscreen == 0 ) 
      {
         fprintf(stderr," Opening display ...\n");
         if ((display = XOpenDisplay(NULL)) != NULL ) 
         {
           wi  = *x       ;
           he = *y       ;
           iHConv = xv[1];
           BackStore = yv[1]; 
           kkprog = xv[2];
           widb = DisplayWidth(display,0) - BW ;
           heib = DisplayHeight(display,0) - BH ;
           if ( wi > widb ) 
                   wi = widb ;
           else if(wi < kscrx)
		   wi = kscrx;
           if ( he > heib ) 
                   he = heib ;
           else if(he < kscry)
		   he = kscry;
           height = he - deltay;
           width = wi - deltax;
           drawing.x = deltax;
           drawing.y = deltay;
           drawing.width = width;
           drawing.height = height - iHConv;
           all.x = 0;
           all.y = 0;
           all.width = wi;
           all.height = he;
   
           attr.backing_store = Always;           /* Redraw when Obscured */
           attr.background_pixel = BlackPixel(display,0);
           mwindow = XCreateWindow(
                     display,
                     RootWindow(display,DefaultScreen(display)),
                     0,0,
                     wi,he,
                     0,CopyFromParent,CopyFromParent,CopyFromParent,
                     CWBackingStore|CWBackPixel,&attr);
   
       depthW  = DisplayPlanes(display, 0);          /* Color Planes */
   
                     
      if ( BackStore == 2 )
  BackStore = DoesBackingStore(ScreenOfDisplay(display,DefaultScreen(display)));
      fprintf(stderr," Backing Store = %d ...\n", BackStore); 
   
      if ( BackStore == 2 )
   
         {
   
          SavedArea1 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
           SavedArea2 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
          SavedArea3 = XCreateWindow(
                        display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
          SavedArea4 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
          SavedArea5 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
          SavedArea6 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
          SavedArea7 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
           SavedArea8 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
          SavedArea9 = XCreateWindow(
                       display,
                       RootWindow(display,DefaultScreen(display)),
                       0,0,
                       widb,heib,
                       0,CopyFromParent,CopyFromParent,CopyFromParent,
                       CWBackingStore|CWBackPixel,&attr);
         }
   
        else
   
         {
   
          SavedPixM1 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM2 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM3 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM4 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM5 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM6 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM7 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM8 = XCreatePixmap( display, mwindow, widb, heib, depthW);
          SavedPixM9 = XCreatePixmap( display, mwindow, widb, heib, depthW);
         }
       
   
   /*      Set title of window */
   
	          XStoreName(display, mwindow, string);
   
   /*      No Resize      */
   
           hints.flags = PPosition | PSize | PMinSize | PMaxSize ;
           hints.x = 0;
           hints.y = 0;
           hints.width = wi;
           hints.height = he;
           hints.min_width = wi;
           hints.min_height = he;
           hints.max_width = wi;
           hints.max_height = he;
           XSetSizeHints(display,mwindow,&hints,XA_WM_NORMAL_HINTS);

   /* 	Set Icon string    */

           if ( kkprog == 1 )
	      {
                XSetIconName(display,mwindow,"SIR97");
              }
	   else
	      {
                XSetIconName(display,mwindow,"EXPO");
              }

   /* 	Set Icon Pixmap    */
           if ( kkprog == 1 )
	      {
    	        Icona = XCreateBitmapFromData(display, mwindow, 
                         icosir_bits, icosir_width, icosir_height);
              }
	   else
	      {
    	        Icona = XCreateBitmapFromData(display, mwindow, 
                         icoexp_bits, icoexp_width, icoexp_height);
              }

	   wmhints.flags = IconPixmapHint;
	   wmhints.icon_pixmap = Icona;
	   XSetWMHints(display, mwindow, &wmhints);

   /* 	    Map Window     */

           XMapWindow(display,mwindow);
   
           /* create graphic context */
   
           values.foreground = WhitePixel(display,0);
           values.background = BlackPixel(display,0);
      
           gc = XCreateGC(display,mwindow,GCForeground|GCBackground,&values);
   
           ClipX = 0;
           ClipY = 0;
           ClipN = 1;
           XSetClipRectangles(display,gc,ClipX,ClipY,&all,ClipN,Unsorted);
           sprintf(messaggio,
                   "Selecting fonts. It could take some minutes. Please wait...\0") ;
           XDrawString(display, mwindow, gc, 8, he-4, messaggio,
                       strlen(messaggio) );
      
      
   
           /* load fonts  */
   
        chfont = 0;
	fp = fopen(nome, "r");
        if (fp == NULL)
	{
		fprintf(stderr,"Error in open file \n");
                return;
        }
        for (np=1;np<=4;np++)
	     fgets(rigar,MAX_FONTLEN,fp);
	fgets(rigar,MAX_FONTLEN,fp);
        for(np=0;np<MAX_FONTLEN;np++)
             if(rigar[np] == '\n') rigar[np] = '\0';
	strcpy(fontp,rigar);
	fgets(rigar,MAX_FONTLEN,fp);
        for(np=0;np<MAX_FONTLEN;np++)
             if(rigar[np] == '\n') rigar[np] = '\0';
	strcpy(fontn,rigar);
	if(fgets(rigar,MAX_FONTLEN,fp) == NULL) 
	{
	   fclose(fp);
/*         crea_listafonts(nome);  inibisco la creazione della lista */
        }
	else
	   fclose(fp);

        if(strstr(fontp,"DEFAULT") != NULL)
	{
		fprintf(stderr," Loading system font \n");
           	fontStruct = LoadFont(fontp,&lar,&alt,&larmin); 
                ifTwoFonts = 0;
        }
       else if((strstr(fontp,"NOFONT") != NULL) || (strcasecmp(fontp,"*") == 0))
	{
		fprintf(stderr,"No font specified \n");
                ifTwoFonts = 0;
/*                strcpy(fontp,"timb*");*/
                strcpy(fontp,"*tim*");
                fontStruct = LoadFontGen(fontp,&lar,&alt,&larmin,1,PITCHP); 
                if(fontStruct == NULL)
	        {
                     strcpy(fontp,"*");
                     fontStruct = LoadFontGen(fontp,&lar,&alt,&larmin,1,PITCHP); 
                     if(fontStruct == NULL)
	             {
			fprintf(stderr," Loading system font \n");
                        strcpy(fontp,"DEFAULT");
           		fontStruct = LoadFont(fontp,&lar,&alt,&larmin); 
                	ifTwoFonts = 0;
                     }
		     else
	             {
                        ifTwoFonts = 1;
		        XSetFont(display, gc, fontStruct->fid);
                     }
                }
		else
	        {
                     ifTwoFonts = 1;
		     XSetFont(display, gc, fontStruct->fid);
                }
		chfont = 1;
        }
	else if(index(fontp,'*') != NULL)
	{
		fprintf(stderr," Loading font %s\n",fontp);
                ifTwoFonts = 0;
                fontStruct = LoadFontGen(fontp,&lar,&alt,&larmin,1,PITCHP); 
                if(fontStruct == NULL)
	        {
		        fprintf(stderr,"Error in LoadFont\n");
			fprintf(stderr," Loading system font \n");
                        strcpy(fontp,"DEFAULT");
           		fontStruct = LoadFont(fontp,&lar,&alt,&larmin); 
                	ifTwoFonts = 0;
                }
		else
	        {
                     ifTwoFonts = 1;
		     XSetFont(display, gc, fontStruct->fid);
                }
		chfont = 1;
        }
	else
	{
                ifTwoFonts = 0;
		fprintf(stderr," Loading font %s\n",fontp);
           	fontStruct = LoadFont(fontp,&lar,&alt,&larmin); 
                if(fontStruct == NULL)
	        {
		        fprintf(stderr,"Error in LoadFont\n");
			fprintf(stderr," Loading system font \n");
                        strcpy(fontp,"DEFAULT");
           		fontStruct = LoadFont(fontp,&lar,&alt,&larmin); 
                	ifTwoFonts = 0;
/*  MR                  chfont = 1;         */
                }
		else
	        {
                     ifTwoFonts = 1;
		     XSetFont(display, gc, fontStruct->fid);
                }
        }


        if(strstr(fontn,"DEFAULT") != NULL)
	{
		fprintf(stderr," Loading system font \n");
           	fontViewer = LoadFont(fontn,&larb,&altb,&larmin); 
                ifTwoFonts = 0;
        }
       else if((strstr(fontn,"NOFONT") != NULL) || (strcasecmp(fontn,"*") == 0))
	{

		fprintf(stderr,"No font specified \n");
                ifTwoFonts = 0;
/*                strcpy(fontn,"courb*");*/
                strcpy(fontn,"*cour*");
                fontViewer = LoadFontGen(fontn,&larb,&altb,&larmin,0,PITCHNP); 
                if(fontViewer == NULL)
	        {
                     strcpy(fontn,"*");
                  fontViewer = LoadFontGen(fontn,&larb,&altb,&larmin,0,PITCHNP); 
                     if(fontViewer == NULL)
	             {
			fprintf(stderr," Loading system font \n");
                        strcpy(fontn,"DEFAULT");
           		fontViewer = LoadFont(fontn,&larb,&altb,&larmin); 
                	ifTwoFonts = 0;
                     }
		     else
                        ifTwoFonts = 1;
                }
		else
                     ifTwoFonts = 1;
		chfont = 1;
        }
	else if(index(fontn,'*') != NULL)
	{
		fprintf(stderr," Loading font %s\n",fontn);
                ifTwoFonts = 0;
                fontViewer = LoadFontGen(fontn,&larb,&altb,&larmin,0,PITCHNP); 
                if(fontViewer == NULL)
	        {
		        fprintf(stderr,"Error in LoadFont\n");
			fprintf(stderr," Loading system font \n");
                        strcpy(fontn,"DEFAULT");
           		fontViewer = LoadFont(fontn,&larb,&altb,&larmin); 
                	ifTwoFonts = 0;
                }
		else
                     ifTwoFonts = 1;
		chfont = 1;
        }
	else
	{
                ifTwoFonts = 0;
		fprintf(stderr," Loading font %s\n",fontn);
           	fontViewer = LoadFont(fontn,&larb,&altb,&larmin); 
                if(fontViewer == NULL)
	        {
		        fprintf(stderr,"Error in LoadFont\n");
			fprintf(stderr," Loading system font \n");
                        strcpy(fontn,"DEFAULT");
           		fontViewer = LoadFont(fontn,&larb,&altb,&larmin); 
                	ifTwoFonts = 0;
/*  MR                  chfont = 1;         */
                }
		else
                     ifTwoFonts = 1;
        }

        if(chfont == 1) update_listafonts(nome);

           larSaveP = lar;
           altSaveP = alt;
           larSaveN = larb;
           altSaveN = altb;
           ifProportional  = 1;
   
            /*     Save title in Titolo  */
   
            strcpy(Titolo,string);
      
            xv[0]  = wi;
            yv[0]  = he;
            xv[1]  = lar;
            yv[1]  = alt;
            xv[2]  = Ascent;
            yv[2]  = Descent;
            xv[3]  = BackStore;
            xv[4]  = widb ;
            yv[4]  = heib;
            kscrx  = wi;
            kscry  = he;
            fprintf(stderr," Backing Store = %d\n", BackStore); 
   
            Shape = XC_watch;
            CursorID = XCreateFontCursor(display,Shape);
      
            VP_x = 0;
            VP_y = 0;
            VP_w = kscrx;
            VP_h = kscry - iHConv;
            jscreen = 1;
            XFlush(display);
            XFlush(display);
         }

         else
         {
            fprintf(stderr,"\n  x11intf: Cannot open DISPLAY \n\n");
            *z = -1;
         }

      }
      else
      {
/*      fprintf(stderr,"\n  resize window                \n\n");  */
            kscrx = xv[0];
            kscry = yv[0];
            hints.flags = PPosition | PSize | PMinSize | PMaxSize ;
            hints.x = 0;
            hints.y = 0;
            hints.width = kscrx;
            hints.height = kscry;
            hints.min_width = kscrx;
            hints.min_height = kscry;
            hints.max_width = kscrx;
            hints.max_height = kscry;

            XSetSizeHints(display,mwindow,&hints,XA_WM_NORMAL_HINTS);

            XResizeWindow(display,mwindow,kscrx,kscry);

            xv[0]  = kscrx;
            yv[0]  = kscry;
            xv[1]  = larSaveP;
            yv[1]  = altSaveP;
            xv[2]  = 0;
            yv[2]  = 0;
            xv[3]  = BackStore;
            xv[4]  = widb ;
            yv[4]  = heib;
      }
      break;

   case 2:                               /* draw to x,y          */
      XSetForeground(display, gc, colors[*col]);
      start_xlx = xv[0];
      start_yly = yv[0];
      xlx       = xv[1];
      yly       = yv[1];
      dx  = start_xlx - xlx;
      dy  = start_yly - yly;
      if (dx == 0 && dy == 0)
         XDrawPoint(display, mwindow, gc, xlx, yly);
      else
         XDrawLine(display, mwindow, gc, xlx, yly,start_xlx,start_yly);

      break;

   case 3:                  /* Rings the bell */
  
      XBell(display,100);
      break;

   case 4:                                  /* manage the mouse */
      Sorcio_C (&ix,&iy,&MouseExitCode);
      *z = MouseExitCode;
      if ( MouseExitCode != 0 )
         {
           mouse_x = ix;
           mouse_y = iy; 
           xv[0] = ix;
           yv[0] = iy;
         }
      else                                 /* non c'era prima */
         {
           xv[0] = mouse_x;
           yv[0] = mouse_y;
         }
      break;


   case 5:                               /* define colors        */
  
      DefColors_new(xv,*z);

  
      break;
   case 6:                     /* clear screen         */

      XSetForeground(display, gc, black);
      if ( *z  == 1 )
            XFillRectangle(display,mwindow,gc,VP_x,VP_y,VP_w,VP_h); 
          else
            XFillRectangle(display,mwindow,gc,0,0,kscrx,kscry); 
      XFlush(display);
      break;

   case 7:                    /* flush buffer */
      XFlush(display);
      break;

   case 8:                    /* Define ViewPort */
      VP_x = xv[0];
      VP_y = yv[0];
      VP_w = xv[1] - xv[0];
      VP_h = yv[1] - yv[0];
   
      drawing.x = VP_x;
      drawing.y = VP_y;
      drawing.width = VP_w;
      drawing.height = VP_h;
      XSetClipRectangles(display,gc,ClipX,ClipY,&drawing,ClipN,Unsorted);
   
      break;

   case 9:                     /* draw a string*/

      XSetBackground(display, gc, black);
      yb = yv[0];
      xb = xv[0];
      if ( xb > 0 )
         {
         xb = xb +          deltax/2; 
         PrintImageMess(xb,yb,string);
         }

      else

      {
         xb = - xb          + deltax/2;
         PrintImageMess(xb,yb,string);
         XSetForeground(display, gc, colors[*col]);
         j=8;
         xb = xb - 30      ;
         asc = Ascent / 3;
         yb = *y - asc;
         points[0].x = xb;
         points[0].y = yb+asc;
         points[1].x = xb+8;
         points[1].y = yb+asc;
         points[2].x = xb+8;
         points[2].y = yb+2*asc;
         points[3].x = xb+15;
         points[3].y = yb;
         points[4].x = xb+8;
         points[4].y = yb-2*asc;
         points[5].x = xb+8;
         points[5].y = yb-asc;
         points[6].x = xb;
         points[6].y = yb-asc;
         points[7].x = xb;
         points[7].y = yb+asc;
         XFillPolygon(display,mwindow, gc, points, j,Nonconvex,CoordModeOrigin);
         XSetForeground(display, gc, colors[old_color]);
      }
      break;

   case 10:                /* draw a circle*/
      XSetForeground(display, gc, colors[*col]);
      XDrawArc(display, mwindow, gc, *x, *y , *z ,*z ,Angle1,Angle2);
      break;

   case 11:                /* fill a circle*/
      /* Angle1,Angle2) = (starting degree, degrees in anti-clockwise) */
      XSetForeground(display, gc, colors[*col1]);
      XFillArc(display, mwindow, gc, *x,*y , *z ,*z ,Angle1,Angle2);
      if ( *col != *col1 ) 
          {
           XSetForeground(display, gc, colors[*col]);
           XDrawArc(display, mwindow, gc, *x, *y , *z ,*z ,Angle1,Angle2);
          }
      break;

   case 12:                /* set ForeGround color */
      XSetForeground(display, gc, colors[*col]);
      old_color = *col;
      break;

   case 13:                /* draw z-points polygon using color col */
      new_color = *col;
      XSetForeground(display, gc, colors[new_color]);
      for (np=1;np<*z;np++)
         XDrawLine(display, mwindow, gc, xv[np-1], yv[np-1], xv[np], yv[np]);
      XSetForeground(display, gc, colors[old_color]);
      break;

   case 14:               /* fill z-points polygon using color col */
      new_color = *col;
      XSetForeground(display, gc, colors[new_color]);
      for (np=0;np<*z;np++)
      {
         points[np].x = xv[np];
         points[np].y = yv[np];
      }
      XFillPolygon(display, mwindow, gc, points, *z,Nonconvex,CoordModeOrigin);
      break;

   case 15:               /* Print a message at the bottom of the window */
      XSetForeground(display, gc, colors[*col1]);
      if ( itwas31 == 1 )
         XFillRectangle(display,mwindow,gc,3,itwas31y+4,kscrx-26,iHConv-6); 
      else
         XFillRectangle(display,mwindow,gc,3,kscry-iHConv+3,kscrx-200,iHConv-6); 
      Bottom(string,*col);
      break;

   case 16:               /* Store atomic species in array Species */

      NumSpec = *z;
      strcpy(Species,string);
      strcpy(Species_copy,string);
      break;

   case 17:             /* Draw rectangle with col filled with col1 */

      
      xb = xv[0];
      yb = yv[0];
      tsx = xv[1] - xv[0];
      tsy = yv[1] - yv[0];
      if ( *col1  >= 0 )
         {
          XSetForeground(display, gc, colors[*col1]);
          XFillRectangle(display,mwindow,gc,xb+1,yb+1,tsx-1,tsy-1);
         }
      XSetForeground(display, gc, colors[*col]);
      XDrawRectangle(display,mwindow,gc,xb,yb,tsx,tsy);

      break;

   case 18:      /* Remove a file */

      break;

   case 19:               /* Save/Restore Area#1   */
      
      if ( *z  == 1 )
         {
          Saved1_x = xv[0];
          Saved1_y = yv[0];
          Saved1_w = xv[1] - xv[0] + 1;
          Saved1_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea1,gc,Saved1_x,Saved1_y,Saved1_w,Saved1_h,0,0);
         }
      else
         {
          Saved1_x = xv[0];
          Saved1_y = yv[0];
          XCopyArea(display,SavedArea1,mwindow,gc,0,0,Saved1_w,Saved1_h,Saved1_x,Saved1_y);
         }
      break;

   case 20:               /* Save/Restore Area#2   */
      
      if ( *z  == 1 )
         {
          Saved2_x = xv[0];
          Saved2_y = yv[0];
          Saved2_w = xv[1] - xv[0] + 1;
          Saved2_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea2,gc,Saved2_x,Saved2_y,Saved2_w,Saved2_h,0,0);
         }
        else
         {
          Saved2_x = xv[0];
          Saved2_y = yv[0];
          XCopyArea(display,SavedArea2,mwindow,gc,0,0,Saved2_w,Saved2_h,Saved2_x,Saved2_y);
         }
      break;

   case 21:               /* Save/Restore Area#3   */
      
      if ( *z  == 1 )
         {
          Saved3_x = xv[0];
          Saved3_y = yv[0];
          Saved3_w = xv[1] - xv[0] + 1;
          Saved3_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea3,gc,Saved3_x,Saved3_y,Saved3_w,Saved3_h,0,0);
         }
        else
         {
          Saved3_x = xv[0];
          Saved3_y = yv[0];
          XCopyArea(display,SavedArea3,mwindow,gc,0,0,Saved3_w,Saved3_h,Saved3_x,Saved3_y);
         }
      break;

   case 22:               /* Save/Restore Area#4   */
      
      if ( *z  == 1 )
         {
          Saved4_x = xv[0];
          Saved4_y = yv[0];
          Saved4_w = xv[1] - xv[0] + 1;
          Saved4_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea4,gc,Saved4_x,Saved4_y,Saved4_w,Saved4_h,0,0);
         }
        else
         {
          Saved4_x = xv[0];
          Saved4_y = yv[0];
          XCopyArea(display,SavedArea4,mwindow,gc,0,0,Saved4_w,Saved4_h,Saved4_x,Saved4_y);
         }
      break;

   case 23:               /* Save/Restore Area#5   */
      
      if ( *z  == 1 )
         {
          Saved5_x = xv[0];
          Saved5_y = yv[0];
          Saved5_w = xv[1] - xv[0] + 1;
          Saved5_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea5,gc,Saved5_x,Saved5_y,Saved5_w,Saved5_h,0,0);
         }
        else
         {
          Saved5_x = xv[0];
          Saved5_y = yv[0];
          XCopyArea(display,SavedArea5,mwindow,gc,0,0,Saved5_w,Saved5_h,Saved5_x,Saved5_y);
         }
      break;

   case 24:               /* Save/Restore Area#6   */
      
      if ( *z  == 1 )
         {
          Saved6_x = xv[0];
          Saved6_y = yv[0];
          Saved6_w = xv[1] - xv[0] + 1;
          Saved6_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea6,gc,Saved6_x,Saved6_y,Saved6_w,Saved6_h,0,0);
         }
        else
         {
          Saved6_x = xv[0];
          Saved6_y = yv[0];
          XCopyArea(display,SavedArea6,mwindow,gc,0,0,Saved6_w,Saved6_h,Saved6_x,Saved6_y);
         }
      break;

   case 25:               /* Save/Restore Area#7   */
      
      if ( *z  == 1 )
         {
          Saved7_x = xv[0];
          Saved7_y = yv[0];
          Saved7_w = xv[1] - xv[0] + 1;
          Saved7_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea7,gc,Saved7_x,Saved7_y,Saved7_w,Saved7_h,0,0);
         }
        else
         {
          Saved7_x = xv[0];
          Saved7_y = yv[0];
          XCopyArea(display,SavedArea7,mwindow,gc,0,0,Saved7_w,Saved7_h,Saved7_x,Saved7_y);
         }
      break;

   case 26:               /* Save/Restore Area#8   */
      
      if ( *z  == 1 )
         {
          Saved8_x = xv[0];
          Saved8_y = yv[0];
          Saved8_w = xv[1] - xv[0] + 1;
          Saved8_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea8,gc,Saved8_x,Saved8_y,Saved8_w,Saved8_h,0,0);
         }
        else
         {
          Saved8_x = xv[0];
          Saved8_y = yv[0];
          XCopyArea(display,SavedArea8,mwindow,gc,0,0,Saved8_w,Saved8_h,Saved8_x,Saved8_y);
         }
      break;

   case 27:               /* Save/Restore Area#9   */
      
      if ( *z  == 1 )
         {
          Saved9_x = xv[0];
          Saved9_y = yv[0];
          Saved9_w = xv[1] - xv[0] + 1;
          Saved9_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedArea9,gc,Saved9_x,Saved9_y,Saved9_w,Saved9_h,0,0);
         }
        else
         {
          Saved9_x = xv[0];
          Saved9_y = yv[0];
          XCopyArea(display,SavedArea9,mwindow,gc,0,0,Saved9_w,Saved9_h,Saved9_x,Saved9_y);
         }
      break;

   case 29:                     /* Connect iz points using color icol */
      new_color = *col;
      XSetForeground(display, gc, colors[new_color]);
      for (np=0;np<*z;np++)
      {
         points[np].x = xv[np];
         points[np].y = yv[np];
      }
      XDrawLines(display, mwindow, gc, points, *z, CoordModeOrigin);
/*    XSetForeground(display, gc, colors[old_color]); */

      break;

   case 30:                     /* Returns the width of a string in pixel*/

      if ( ifProportional  == 1 )
         {
           n = XTextWidth(fontStruct, string, *z             );  
         }
        else
         {
           n = XTextWidth(fontViewer, string, *z             );  
         }

      xv[0] = n;

      break;


   case 44:                     /* change font */

      if ( ifTwoFonts  == 1 )
         {
           if ( *z  == 1 )
              {
                 XSetFont(display,gc,fontStruct->fid);
                 ifProportional  = 1;
                 xv[0] = larSaveP ;
                 yv[0] = altSaveP ;
              }
             else
              {
                 XSetFont(display,gc,fontViewer->fid);
                 ifProportional  = 0;
                 xv[0] = larSaveN ;
                 yv[0] = altSaveN ;
              }
         }
         {
           xv[0] = larSaveP ;
           yv[0] = altSaveP ;
         }

      break;

   case 61:               /* Save/Restore Area#1   */
      
      if ( *z  == 1 )
         {
          Saved1_x = xv[0];
          Saved1_y = yv[0];
          Saved1_w = xv[1] - xv[0] + 1;
          Saved1_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM1,gc,Saved1_x,Saved1_y,Saved1_w,Saved1_h,0,0);
         }
        else
         {
          Saved1_x = xv[0];
          Saved1_y = yv[0];
          XCopyArea(display,SavedPixM1,mwindow,gc,0,0,Saved1_w,Saved1_h,Saved1_x,Saved1_y);
         }
      break;

   case 62:               /* Save/Restore Area#2   */
      
      if ( *z  == 1 )
         {
          Saved2_x = xv[0];
          Saved2_y = yv[0];
          Saved2_w = xv[1] - xv[0] + 1;
          Saved2_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM2,gc,Saved2_x,Saved2_y,Saved2_w,Saved2_h,0,0);
         }
        else
         {
          Saved2_x = xv[0];
          Saved2_y = yv[0];
          XCopyArea(display,SavedPixM2,mwindow,gc,0,0,Saved2_w,Saved2_h,Saved2_x,Saved2_y);
         }
      break;

   case 63:               /* Save/Restore Area#3   */
      
      if ( *z  == 1 )
         {
          Saved3_x = xv[0];
          Saved3_y = yv[0];
          Saved3_w = xv[1] - xv[0] + 1;
          Saved3_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM3,gc,Saved3_x,Saved3_y,Saved3_w,Saved3_h,0,0);
         }
        else
         {
          Saved3_x = xv[0];
          Saved3_y = yv[0];
          XCopyArea(display,SavedPixM3,mwindow,gc,0,0,Saved3_w,Saved3_h,Saved3_x,Saved3_y);
         }
      break;

   case 64:               /* Save/Restore Area#4   */
      
      if ( *z  == 1 )
         {
          Saved4_x = xv[0];
          Saved4_y = yv[0];
          Saved4_w = xv[1] - xv[0] + 1;
          Saved4_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM4,gc,Saved4_x,Saved4_y,Saved4_w,Saved4_h,0,0);
         }
        else
         {
          Saved4_x = xv[0];
          Saved4_y = yv[0];
          XCopyArea(display,SavedPixM4,mwindow,gc,0,0,Saved4_w,Saved4_h,Saved4_x,Saved4_y);
         }
      break;

   case 65:               /* Save/Restore Area#5   */
      
      if ( *z  == 1 )
         {
          Saved5_x = xv[0];
          Saved5_y = yv[0];
          Saved5_w = xv[1] - xv[0] + 1;
          Saved5_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM5,gc,Saved5_x,Saved5_y,Saved5_w,Saved5_h,0,0);
         }
        else
         {
          Saved5_x = xv[0];
          Saved5_y = yv[0];
          XCopyArea(display,SavedPixM5,mwindow,gc,0,0,Saved5_w,Saved5_h,Saved5_x,Saved5_y);
         }
      break;

   case 66:               /* Save/Restore Area#6   */
      
      if ( *z  == 1 )
         {
          Saved6_x = xv[0];
          Saved6_y = yv[0];
          Saved6_w = xv[1] - xv[0] + 1;
          Saved6_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM6,gc,Saved6_x,Saved6_y,Saved6_w,Saved6_h,0,0);
         }
        else
         {
          Saved6_x = xv[0];
          Saved6_y = yv[0];
          XCopyArea(display,SavedPixM6,mwindow,gc,0,0,Saved6_w,Saved6_h,Saved6_x,Saved6_y);
         }
      break;

   case 67:               /* Save/Restore Area#7   */
      
      if ( *z  == 1 )
         {
          Saved7_x = xv[0];
          Saved7_y = yv[0];
          Saved7_w = xv[1] - xv[0] + 1;
          Saved7_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM7,gc,Saved7_x,Saved7_y,Saved7_w,Saved7_h,0,0);
         }
        else
         {
          Saved7_x = xv[0];
          Saved7_y = yv[0];
          XCopyArea(display,SavedPixM7,mwindow,gc,0,0,Saved7_w,Saved7_h,Saved7_x,Saved7_y);
         }
      break;

   case 68:               /* Save/Restore Area#8   */
      
      if ( *z  == 1 )
         {
          Saved8_x = xv[0];
          Saved8_y = yv[0];
          Saved8_w = xv[1] - xv[0] + 1;
          Saved8_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM8,gc,Saved8_x,Saved8_y,Saved8_w,Saved8_h,0,0);
         }
        else
         {
          Saved8_x = xv[0];
          Saved8_y = yv[0];
          XCopyArea(display,SavedPixM8,mwindow,gc,0,0,Saved8_w,Saved8_h,Saved8_x,Saved8_y);
         }
      break;

   case 69:               /* Save/Restore Area#9   */
      
      if ( *z  == 1 )
         {
          Saved9_x = xv[0];
          Saved9_y = yv[0];
          Saved9_w = xv[1] - xv[0] + 1;
          Saved9_h = yv[1] - yv[0] + 1;
          XCopyArea(display,mwindow,SavedPixM9,gc,Saved9_x,Saved9_y,Saved9_w,Saved9_h,0,0);
         }
        else
         {
          Saved9_x = xv[0];
          Saved9_y = yv[0];
          XCopyArea(display,SavedPixM9,mwindow,gc,0,0,Saved9_w,Saved9_h,Saved9_x,Saved9_y);
         }
      break;

   case 80:                     /* change cursor shape */

      if ( *z  == 0 )
            XUndefineCursor(display,mwindow);
      else
            XDefineCursor(display,mwindow,CursorID);
      break;

   case 81:          /* change name to the window */

      XStoreName(display, mwindow, string);
      break;

   case 90:                     /* draw a string*/

      xb = xv[0];
      yb = yv[0] + 10;
            C_PrintMess(xb,yb,string,*col);
      break;


   default:               /* big trouble          */
      fprintf(stderr,"Error in x11intf action= %d", *action);
      break;

   }
}
/*-------------------------------------------------------------*/
/*   x11intf1             Fast routine to draw segments        */
/*-------------------------------------------------------------*/
/*   CHANGE       for Ultrix or Silicon Graphics or Linux      */
/*   void xwinnew_(..........)                                 */
/*   OTHERWISE                                                 */
/*   void xwinnew(..........)                                  */
/*-------------------------------------------------------------*/
void xwinnew(xv1,yv1,xv2,yv2,xv3,yv3,col1,col2,z)
int xv1[],yv1[],xv2[],yv2[],xv3[],yv3[],col1[],col2[],*z;
{

   int np, new_color, old_color;

      old_color = 0;
      for (np=0;np<*z;np++)
      {
         new_color = col1[np];
         if ( new_color != old_color ) 
         {
           old_color = new_color;
           XSetForeground(display, gc, colors[new_color]);
         }
         XDrawLine(display, mwindow, gc, xv1[np], yv1[np], xv2[np], yv2[np]);
         new_color = col2[np];
         if ( new_color != old_color ) 
         {
           old_color = new_color;
           XSetForeground(display, gc, colors[new_color]);
         }
         XDrawLine(display, mwindow, gc, xv2[np], yv2[np], xv3[np], yv3[np]);
      }

}

/*-------------------------------------------------------------*/
/*   sirtm                        Time & date routine          */
/*-------------------------------------------------------------*/
/*   CHANGE       for Ultrix or Silicon Graphics or Linux      */
/*   void sirtm_(datestring)                                   */
/*   OTHERWISE                                                 */
/*   void sirtm(datestring)                                    */
/*-------------------------------------------------------------*/
#include <time.h>
void sirtm(datestring)
char *datestring;
{
        struct timeval tv;
	time_t timer;
        gettimeofday(&tv,NULL);
        timer = tv.tv_sec;
        strcpy(datestring, asctime(localtime(&timer)));
/*      strftime(datestring,19,"%d.%m.%y %H:%M:%S", localtime(&timer)); */
}
/*-------------------------------------------------------------*/
/*   mrclock                      Clock routine                 */
/*-------------------------------------------------------------*/
/*   CHANGE       for Ultrix or Silicon Graphics or Linux      */
/*   int mrclock_()                                            */
/*   OTHERWISE                                                 */
/*   int mrclock()                                             */
/*-------------------------------------------------------------*/
int mrclock()
{
   int tempo, sec;
   struct timeval tv;
   gettimeofday(&tv,NULL);
   sec = tv.tv_sec;
   if(sec > 1e6)
       sec = sec - (int)(sec/1e6)*1e6;
   tempo = sec*100 + tv.tv_usec/10000;
   return (tempo);
}
/*-------------------------------------------------------------*/
/*   sirchdir                     Change Directory routine      */
/*-------------------------------------------------------------*/
/*   CHANGE       for Ultrix or Silicon Graphics or Linux      */
/*   void sirchdir_(newdir, ier)                               */
/*   OTHERWISE                                                 */
/*   void sirchdir(newdir, ier)                                */
/*-------------------------------------------------------------*/
void sirchdir(newdir, ier)
int ier;
char *newdir;
{
        ier = chdir(newdir);
}
/*-------------------------------------------------------------*/
/*   DefColors_new: definisce i colori utilizzati dal programma*/
/*   nel caso di schermo a colori o monocromatico.             */
/*-------------------------------------------------------------*/
DefColors_new(xv,iz)
int xv[],iz;
{
   typedef struct {
                    int red;
                    int green;
                    int blue;
                   }  rgbcol ;
   int i,class;
   XColor rgb;
   XVisualInfo template,*info;
   Colormap cmap;

    cmap   = DefaultColormap(display, 0);

    Monocro = False;

    template.screen = DefaultScreen(display);
    template.depth  = depthW;
 
    info = XGetVisualInfo(display,VisualScreenMask | VisualDepthMask,
           &template,&i);
    if(i > 0)
         class = info[0].class;
    XFree(info);

   fprintf(stderr," ... done! Depth = %d\n",depthW);
   
    if(depthW >= 4)                     /* >= 16 Colors or GrayScale */
    {
       for (i = 0; i < MAX_COLORS; i++ )
       {
          rgb.red   =  xv[i*3  ];
          rgb.green =  xv[i*3+1];
          rgb.blue  =  xv[i*3+2];
          if(XAllocColor(display,cmap,&rgb) == 1)
               colors[i] = rgb.pixel;
          else
            {
               fprintf(stderr," Cannot allocate color %d \n",i);
               colors[i] = i;
            }
       }
    }
    if ((depthW < 4) || Monocro)
    {
        colors[0] = BlackPixel(display,0);
        for ( i = 1; i < MAX_COLORS; i++ )
           colors[i] = WhitePixel(display, 0);
        fprintf(stderr,"Allocated 2 Colors on Monochrome Display\n");
    }
    black = colors[0];
    white = colors[MAX_COLORS-1];
}

/*------------------------------------------------------------------*/
/*    LoadFont: load font                                           */
/*------------------------------------------------------------------*/
XFontStruct *LoadFont(fontnome,lar, alt, larmin)
int *lar, *alt, *larmin;
char *fontnome;
{

	if(strstr(fontnome,"DEFAULT") != NULL)
		fontPunt = XQueryFont(display, XGContextFromGC(gc));
	else 
		fontPunt = XLoadQueryFont(display, fontnome);
	if (fontPunt != NULL)
	{
		*lar = fontPunt->max_bounds.width;
		*larmin = fontPunt->min_bounds.width;
		Ascent = fontPunt->max_bounds.ascent;
		Descent = fontPunt->max_bounds.descent;
		*alt = Ascent + Descent;
                if ( *lar == *larmin )
         		fprintf(stderr," (non proportional font) \n"); 
                else
         		fprintf(stderr," (proportional font) \n"); 
                return (fontPunt);
	}
}				

/*------------------------------------------------------------------*/
/*    LoadFontGen: load fonts                                       */
/*------------------------------------------------------------------*/
XFontStruct *LoadFontGen(fontnomi, lar, alt, larmin,fprop,dimensione)
int *lar, *alt, *larmin,fprop,dimensione;
char *fontnomi;
{
	char **fontnames;
	Boolean SearchFontGen();

	if(SearchFontGen(fontnomi, dimensione,fprop))
	{
		*lar = fontPunt->max_bounds.width;
		*larmin = fontPunt->min_bounds.width;
		Ascent = fontPunt->max_bounds.ascent;
		Descent = fontPunt->max_bounds.descent;
		*alt = Ascent + Descent;
                if ( *lar == *larmin )
         		fprintf(stderr," (non proportional font) \n"); 
                else
         		fprintf(stderr," (proportional font) \n"); 
        	ifTwoFonts = 1;
		return (fontPunt);
	}
	else
 		return (NULL);

}
/*------------------------------------------------------------------*/
/*    SearchFontGen: looks for possible fonts                       */
/*------------------------------------------------------------------*/
Boolean SearchFontGen(fontnomi, dimensione,fprop)
char *fontnomi;
int dimensione,fprop;
{	
	int i, larmax,  larmin, nFound, jFont, fonts[MAX_FONTNUM][2],count;
	unsigned long w,ps;
	char **fontnames;

		fprintf(stderr," Searching fonts %s \n",fontnomi);
	fontnames = XListFonts(display, fontnomi, MAX_FONTNUM, &count);	
		fprintf(stderr," Found %d fonts \n",count);
	nFound = 0;
   	for(i=0;i<count;i++)
		if((strstr(fontnames[i],"symb") == NULL) && 
           	(strstr(fontnames[i],"curs") == NULL))
		{
			if((fontPunt = XLoadQueryFont(display, fontnames[i])) != NULL)
			{
				XGetFontProperty(fontPunt, XA_WEIGHT, &w);
				XGetFontProperty(fontPunt, XA_POINT_SIZE, &ps);
				ps /=10;
				larmax = fontPunt->max_bounds.width;
				larmin = fontPunt->min_bounds.width;
                                if(((fprop == 0) && (larmax == larmin))  ||
                                  ((fprop == 1) && (larmax != larmin)))
				{
					if(ps == dimensione)
					{
						fonts[nFound][0] = i;
						fonts[nFound][1] = w;
						nFound++;           
					}
				}
			}
		}
		fprintf(stderr," Selected %d fonts \n",nFound);
	if(nFound > 0)       /*  scelgo la fonte con peso maggiore (bold) */
	{
		jFont = 0;
		for(i=1; i<nFound; i++)
			if(fonts[i][1] > fonts[jFont][1]) jFont = i;	
		i = fonts[jFont][0];


		if((fontPunt = XLoadQueryFont(display, fontnames[i])) == NULL)
		{
			XFreeFontNames(fontnames);
			return (False);
		}

		fprintf(stderr, " Found. \n");
                strcpy(fontnomi, fontnames[i]);
		fprintf(stderr, " selected font= %s \n",fontnomi);				

		XFreeFontNames(fontnames);
		return (True);    
	}
	XFreeFontNames(fontnames);
	return (False);                   
}
/*-------------------------------------------------------------*/
/*   C_PrintMess: draws "string" at "xb,yb", color "colo"      */
/*              used to draw keys                              */
/*-------------------------------------------------------------*/
void C_PrintMess(xb,yb,string,colo)
int xb,yb,colo;
char string[];
{
   int n;

   if (colo != -1)
      XSetForeground(display, gc, colors[colo]);
   XDrawString(display, mwindow, gc, xb, yb, string, strlen(string) );
   if (colo != -1)
      XSetForeground(display, gc, colors[old_color]);
   XFlush(display);
}
/*-------------------------------------------------------------*/
/*   PrintImageMess: draws "string" at "xb,yb"                 */
/*-------------------------------------------------------------*/
void PrintImageMess(xb,yb,string)
int xb,yb;
char string[];
{
   int i;
   for(i=strlen(string);i<79;i++)
      string[i] = ' ';
   string[79] = '\0';
   XDrawImageString(display, mwindow, gc, xb       , yb, string, 79 );
   XFlush(display);
}
/*-------------------------------------------------------------*/
/*   Bottom: Print message "string" at the bottom of the window*/
/*-------------------------------------------------------------*/
void Bottom(string,color)
char string[];
int color;
{
   int xb,yb;

   XSetForeground(display, gc, colors[color]);
   xb = 8; 
   if ( itwas31 == 1 )
      yb = itwas31y + 18;
   else
      yb = kscry-6;
   XDrawString(display, mwindow, gc, xb, yb, string, strlen(string) );
   XFlush(display);
}
/*-------------------------------------------------------------*/
/*   Sorcio_C: Manage mouse and keyboard                       */
/*-------------------------------------------------------------*/
void Sorcio_C(ix,iy,iz)
int *ix,*iy,*iz;
{
      static int OldClick;
      char BufferReturn[2];
      int Boh, idum;
      KeySym keysym;

   *iz = 0;
   XSelectInput(display,mwindow,ButtonPressMask|
                                ButtonReleaseMask|
                                ButtonMotionMask|
                                PointerMotionMask|
                                KeyPressMask);
   if (XCheckMaskEvent(display,        ButtonPressMask|
                                ButtonReleaseMask|
                                ButtonMotionMask|
                                PointerMotionMask|
                                KeyPressMask,&event))
   {
      switch(event.type)
      {
             case ButtonPress:
                  switch(event.type)
                  {
                  }
                  *ix = event.xbutton.x;
                  *iy = event.xbutton.y;
                  if ( event.xbutton.button == 1 )
                  {
                      *iz = 1;
                  }
                    else
                  {
                      *iz = -1;
                  }
/* fprintf(stderr," Bottone %d\n", event.xbutton.button); */
                  OldClick = *iz;
                  break;
   
             case ButtonRelease:
   
                  *ix = event.xbutton.x;
                  *iy = event.xbutton.y;
                  *iz = 2;
                  break;
   
             case MotionNotify:
                  *ix = event.xmotion.x;
                  *iy = event.xmotion.y;
                  *iz = 3 * OldClick;
                  break;
   
             case MotionNotify && ButtonPress:
                  *ix = event.xmotion.x;
                  *iy = event.xmotion.y;
                  *iz = 5;
                  if ( event.xbutton.button == 1 )
                  *iz = 5;
                  if ( event.xbutton.button == 2 )
                  *iz = 5;
                  if ( event.xbutton.button == 3 )
                  *iz = 5;
                  break;
   
             case KeyPress:
   
            Boh = XLookupString((XKeyEvent *)&event, 
                                BufferReturn,
                                sizeof (BufferReturn),
                                &keysym,
                                NULL);
            if ( keysym == XK_Left )
                  {
                      *ix = 1004;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_Right)
                  {
                      *ix = 1006;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_Up)
                  {
                      *ix = 1008;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_Down)
                  {
                      *ix = 1002;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_Home)
                  {
                      *ix = 1007;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_End)
                  {
                      *ix = 1001;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_Prior)
                  {
                      *ix = 1009;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_Next)
                  {
                      *ix = 1003;
                      *iz = 4;
                       break;
                  }
            else if ( keysym == XK_Delete)
                  {
                      *ix = 1011;
                      *iz = 4;
                       break;
                  }

                 idum = (int) BufferReturn[0];
                  *ix = idum;
                  if ( Boh == 0 )
                  {
                      *iz = 0;
                  }
                    else
                  {
                      *iz = 4;
                  }
                  break;

                  default:
                  *iz = 0;
                  break;
      }

       XFlush(display);

   }
}
/*-------------------------------------------------------------*/
/*   ClearBottom: clear area at bottom of the window           */
/*-------------------------------------------------------------*/
void ClearBottom()
{
   int i;
   char string[80];

   /* Clear messages area */

   for(i=0;i<79;i++)
      string[i] = ' ';
   string[79] = '\0';
   XDrawImageString(display,mwindow,gc,85,he-4,string,79);

   /* Clear keys area */

   XClearArea(display, mwindow, deltax+1,he-25,0,0,0);
}

void crea_listafonts(nomefile)
char *nomefile;
{	
	int i, larmax,  larmin, count,count1;
	unsigned long w,ps,fam,font;
	char **fontnames;
	FILE *fp;
	int pitch1 = 8, pitch2 = 16;

	fprintf(stderr," Create font list. It could take some minutes .... \n");
	fontnames = XListFonts(display, "*", MAX_FONTNUM, &count);
		fprintf(stderr," Found %d fonts \n",count);
	count1 = 0;
        fp  = fopen(nomefile,"a");
	for(i=0;i<count;i++)
	if((strstr(fontnames[i],"symb") == NULL) && 
           (strstr(fontnames[i],"curs") == NULL))
	{
 	   if((fontPunt = XLoadQueryFont(display, fontnames[i])) != NULL)
	   {
		XGetFontProperty(fontPunt, XA_WEIGHT, &w);
		XGetFontProperty(fontPunt, XA_POINT_SIZE, &ps);
		ps /=10;
		XGetFontProperty(fontPunt, XA_FAMILY_NAME, &fam);
		XGetFontProperty(fontPunt, XA_FONT, &font);
		larmax = fontPunt->max_bounds.width;
		larmin = fontPunt->min_bounds.width;
                if(larmax != larmin)
	        {
		   if((ps >= pitch1) && (ps <= pitch2))
	           {
		      fprintf(fp, "%s %d %d %ld %ld %ld %ld\n",
		      fontnames[i],larmax,larmin,w,ps,fam, font);
		      count1++;
	           }
	        }
	   }
	}
	fclose(fp);
		fprintf(stderr," Selected %d fonts \n",count1);
	XFreeFontNames(fontnames);
}
void update_listafonts(nomefile)
char *nomefile;
{	
	int np;
	FILE *fp;
	char riga1[MAX_FONTLEN];
	char riga2[MAX_FONTLEN];
	char riga3[MAX_FONTLEN];
	char riga4[MAX_FONTLEN];

        fp = fopen(nomefile, "r");
        fgets(riga1,MAX_FONTLEN,fp);
        fgets(riga2,MAX_FONTLEN,fp);
        fgets(riga3,MAX_FONTLEN,fp);
        fgets(riga4,MAX_FONTLEN,fp);
	fclose(fp);
	np = unlink(nomefile);

	fp = fopen(nomefile, "w");
        fputs(riga1,fp);
        fputs(riga2,fp);
        fputs(riga3,fp);
        fputs(riga4,fp);
	fprintf(fp,"%s\n",fontp);
	fprintf(fp,"%s\n",fontn);
	fclose(fp);
/*
	FILE *fp,*fp1;
	char rigar[MAX_FONTLEN];
	int np;

	np = rename(nomefile, "tmp.fnt");
        fp1 = fopen("tmp.fnt","r");
	fp = fopen(nomefile, "w");
        for (np=1;np<=4;np++)
	{	
	     fgets(rigar,MAX_FONTLEN,fp1);
   	     fputs(rigar,fp);
	}
        for (np=1;np<=2;np++)
	     fgets(rigar,MAX_FONTLEN,fp1);
	fprintf(fp,"%s\n",fontp);
	fprintf(fp,"%s\n",fontn);
	while(fgets(rigar,MAX_FONTLEN,fp1) != NULL)
		fputs(rigar,fp);
	fclose(fp);
	fclose(fp1);
	np = unlink("tmp.fnt");
*/
}
